﻿/* =========================================================================

InDesign ExtendScript

NAME: toRTF-v2
VERSION: 2

AUTHOR: Alexander Albert
DATE  : 10.01.2019

DESCRIPTION: Экспорт всех текстов на страницах в rtf-файл
порядок определяется положением текстовых фреймов на странице

Скрипт экспортирует все тексты со страниц документа. 
Неразрывный пробел заменяется на обычный.
Белые буквы красятся черным.
Шаблонные страницы не анализируются.

Порядок определяется положением верхнего левого угла текстовых фреймов на странице.
Сначала постранично,
на странице сверху вниз,
если фреймы примерно на одном уровне (до 3 единиц измерения) то слева направо.
Связанные фреймы по первому фрейму в цепочке.
Если цепочка связанных фреймов переходит на другую страницу, то текст рвется по границе абзаца

В результате создается rtf-файл с именем совпадающем с именем indd-файла

E-MAIL: alb2691@gmail.com

============================================================================ */
main();
function main(){
	//Make certain that user interaction (display of dialogs, etc.) is turned on.
	app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;
	if(app.documents.length != 0){
		if (app.activeDocument.stories.length != 0){
			alert("Экспорт всех текстов на страницах в rtf-файл \r1) Порядок определяется положением текстовых фреймов на странице\r2) Если край фрейма залезает на страницу, то текст из него тоже попадет в rtf-файл");
			myExport();
			alert("Done");
		}
		else{
			alert("The document does not contain any text. Please open a document containing text and try again.");
		}
	}
	else{
		alert("No documents are open. Please open a document and try again.");
	}
}

function myExport() {
    var myDoc = app.activeDocument;
            //создаем временный фрейм
    var myTempTextFrame = myDoc.pages[0].textFrames.add({geometricBounds: [0,-110,200,-10], contents: ""});
    var myTempStory = myTempTextFrame.parentStory;
    
    //цикл по страницам
            var myPreviousFrame = myTempTextFrame;
    for(var i=0; i < myDoc.pages.length; i++) {
                var myPage = myDoc.pages[i];
                var myTextFrames = [];
                //все текстовые фреймы в массив
                for(var j = 0; j < myPage.textFrames.length; j++) {
					if(myPage.textFrames[j].parentPage == myPage) {		// если фрейм касается страницы, то он включен в колекцию, но его parentPage другая (null)
						myTextFrames.push(myPage.textFrames[j])
					}
                }
                //сортировка массива
//				alert("i = " + i + "; j = " + j + "; myPage.textFrames.length = " + myPage.textFrames.length);
                myTextFrames.sort(myCompareFrames);
                
                //копируем тексты из фреймов по порядку в новый фрейм
                //*
                for (var k=0; k < myTextFrames.length; k++) {
                    var myCounterStart = 0;
                    if (myPreviousFrame != myTextFrames[k].previousTextFrame) myTempStory.insertionPoints[-1].contents = "\r"

                    if (myTextFrames[k].previousTextFrame != null) {
                    if(myTextFrames[k].previousTextFrame.paragraphs[-1] == myTextFrames[k].paragraphs[0]) myCounterStart = 1;
                    else myCounterStart = 0;
                    }
                    for (var n = myCounterStart; n < myTextFrames[k].paragraphs.length; n++) {
                        var myPar = myTextFrames[k].paragraphs[n];
                        var myDuplPar = myPar.duplicate(LocationOptions.AT_END, myTempStory);
                        if (myPar.bulletsAndNumberingListType == ListType.NUMBERED_LIST) {
                                myDuplPar.bulletsAndNumberingListType = ListType.NO_LIST;
                                myDuplPar.insertionPoints[0].contents = myPar.bulletsAndNumberingResultText;
                        }
                        if (myPar.bulletsAndNumberingListType == ListType.BULLET_LIST) myDuplPar.convertBulletsAndNumberingToText();
                    }
                            myPreviousFrame = myTextFrames[k];
                } // */
     }
    myRecolorWhiteToBlack(myTempStory);
	myRemoveNonBreakWhitespace(myTempStory);
     //*експорт в ртф
    			myFolder= Folder.selectDialog ("Choose a Folder");
			if(myFolder != null){
                    myFileName = myDoc.name + ".rtf";
                    myFormat = ExportFormat.RTF;
                    myFilePath = myFolder + "/" + myFileName;
                    myFile = new File(myFilePath);
                    myTempStory.exportFile(myFormat, myFile);
			}   //*/
        
        
        //удаляю временный фрейм
        myTempTextFrame.remove();

}

function myRemoveNonBreakWhitespace(myStory)  {
		//убираем неразрывные пробелы (из-за них в rtf-е появляются ???)
	resetFindChangePreferences();
	app.findTextPreferences.properties = {findWhat: "^S"};      //
	app.changeTextPreferences.properties = {changeTo: " "};
	app.findChangeTextOptions.properties = {includeFootnotes:false, includeHiddenLayers:false, includeLockedLayersForFind: false, includeLockedStoriesForFind: false,
 includeMasterPages:false};
	myStory.changeText();                 //замена
	resetFindChangePreferences();
}

function resetFindChangePreferences () {
	app.changeTextPreferences = NothingEnum.nothing;
	app.findTextPreferences = NothingEnum.nothing;
	app.changeGrepPreferences = NothingEnum.nothing;
	app.findGrepPreferences = NothingEnum.nothing;
}

function myRecolorWhiteToBlack(myTempStory) {
	//Clear the find/change text preferences.
	app.findTextPreferences = NothingEnum.nothing;
	app.changeTextPreferences = NothingEnum.nothing;
	//Set the find options.
	app.findChangeTextOptions.caseSensitive = false;
	app.findChangeTextOptions.includeFootnotes = false;
	app.findChangeTextOptions.includeHiddenLayers = false;
	app.findChangeTextOptions.includeLockedLayersForFind = false;
	app.findChangeTextOptions.includeLockedStoriesForFind = false;
	app.findChangeTextOptions.includeMasterPages = false;
	app.findChangeTextOptions.wholeWord = false;
	//Search the document for the string "copy" and change it to "text".
	app.findTextPreferences.fillColor = app.activeDocument.swatches.item("Paper");
	app.changeTextPreferences.fillColor = app.activeDocument.swatches.item("Black");
	myTempStory.changeText();
	//Clear the find/change text preferences after the search.
	app.findTextPreferences = NothingEnum.nothing;
	app.changeTextPreferences = NothingEnum.nothing;
    
}    

function myCompareFrames(frame1, frame2) {
    var DX = 3;             //фреймы выровнены по вертикали если разница Y1-Y2 меньше DX
    var frame1Y = 0;
    var frame2Y = 0;
    
    if (frame1.parentPage != frame2.parentPage) {
	try {
        return frame1.parentPage.documentOffset - frame2.parentPage.documentOffset;
	}
	catch (err) {
//		alert("frame1.parentPage.name " + frame1.parentPage.name);
//		alert("frame2.parentPage.name " + frame2.parentPage.name);
		return 0;
	}
    }
    
    if (frame1.parentStory == frame2.parentStory) {
        for (var i = 0; i < frame1.parentStory.textContainers.length; i++) {
            if (frame1.parentStory.textContainers[i] == frame1) return -1;
            if (frame1.parentStory.textContainers[i] == frame2) return 1;
        }
    }
    switch (frame1.textFramePreferences.verticalJustification) {
        case VerticalJustification.TOP_ALIGN:
            frame1Y = frame1.geometricBounds[0];
            break;
        case VerticalJustification.BOTTOM_ALIGN:
            frame1Y = frame1.geometricBounds[2];
            break;
         default:
            frame1Y = frame1.geometricBounds[0] + (frame1.geometricBounds[2] - frame1.geometricBounds[0])/2;
    }
    switch (frame2.textFramePreferences.verticalJustification) {
        case VerticalJustification.TOP_ALIGN:
            frame2Y = frame2.geometricBounds[0];
            break;
        case VerticalJustification.BOTTOM_ALIGN:
            frame2Y = frame2.geometricBounds[2];
            break;
         default:
            frame2Y = frame2.geometricBounds[0] + (frame2.geometricBounds[2] - frame2.geometricBounds[0])/2;
    }
    if (Math.abs(frame1Y - frame2Y) <= DX) return frame1.geometricBounds[1] - frame2.geometricBounds[1];
    else return frame1Y - frame2Y;
}

